<?php

namespace Drupal\jsnippet\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\jsnippet\JSnippetInterface;
use Drupal\file\Entity\File;

/**
 * Defines the JSnippet entity.
 *
 * @ConfigEntityType(
 *   id = "jsnippet",
 *   label = @Translation("Snippet"),
 *   handlers = {
 *     "list_builder" = "Drupal\jsnippet\Controller\JSnippetListBuilder",
 *     "form" = {
 *       "add" = "Drupal\jsnippet\Form\JSnippetAddForm",
 *       "edit" = "Drupal\jsnippet\Form\JSnippetEditForm",
 *       "delete" = "Drupal\jsnippet\Form\JSnippetDeleteForm",
 *     },
 *        "access" = "Drupal\jsnippet\JSnippetAccessControlHandler",
 *   },
 *   config_prefix = "jsnippet",
 *   admin_permission = "administer snippets",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *   },
 *   links = {
 *     "edit-form" = "/admin/structure/snippet/{jsnippet}",
 *     "delete-form" = "/admin/structure/snippet/{jsnippet}/delete",
 *   }
 * )
 */
class JSnippet extends ConfigEntityBase implements JSnippetInterface {

  const SNIPPET_DIR = 'public://snippets/';

  /**
   * The Snippet ID.
   *
   * @var string
   */
  public $id;

  /**
   * The Snippet label.
   *
   * @var string
   */
  public $label;

  /**
   * The Snippet.
   *
   * @var string
   */
  public $snippet;

  /**
   * The Snippet Scope.
   *
   * @var string
   */
  public $scope;

  /**
   * Save the JSnippet Entity and generate JS files to be used as libraries.
   *
   * {@inheritdoc}.
   */
  public function save() {
    $return = parent::save();
    // Write snippet to a library file if it's not flagged as external.
    if ($this->scope <> 'external') {
      // Fetch an existing file, or build a new file entity.
      if ($files = \Drupal::entityTypeManager()->getStorage('file')->loadByProperties(['uri' => $this->getUri()])) {
        $file = end($files);
      }
      else {
        $file = File::create([
          'uid' => 1,
          'filename' => $this->getFileName(),
          'uri' => $this->getUri(),
          'status' => 1,
        ]);
        $file->save();
      }

      // Write out the behavior to an appropriate file.
      $directory = $this::SNIPPET_DIR;
      if (file_prepare_directory($directory, FILE_CREATE_DIRECTORY)) {
        file_put_contents($file->getFileUri(), $this->getBehavior());
        $file->save();
      }
    }

    $this->trustedData = FALSE;
    return $return;
  }

  /**
   * Get the filename for the snippet based on it's machine name.
   */
  public function getFileName() {
    return $this->id() . '.js';
  }

  /**
   * Get the full URI for the snippet in the public filesystem.
   */
  public function getUri() {
    if ($this->scope <> 'external') {
      $uri = $this::SNIPPET_DIR . $this->getFileName();
    }
    else {
      $uri = trim($this->snippet);
    }
    return $uri;
  }

  /**
   * Build a Drupal Behavior from the snippet.
   */
  public function getBehavior() {
    $behavior = 'Drupal.behaviors.' . $this->id() . ' = {' . PHP_EOL;
    $behavior .= 'attach: function (context, settings) {' . PHP_EOL;
    $behavior .= $this->snippet . PHP_EOL;
    $behavior .= '}};' . PHP_EOL;
    return $behavior;
  }

}
